/*****************************************************************************
 *
 * Timer Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        Timer.c
 * Dependencies:    
 * Processor:       
 * Compiler:       	
 * Linker:          
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar		  30/June/06	     ...	
 * Brant Ivey			7/3/07		Updated for v4.0 of class
 *****************************************************************************/

#include <p24FJ128GA010.h>

_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON &  FNOSC_FRC & POSCMOD_NONE)


//Function Prototypes
unsigned char GetKey();
void Display(char, char, char, char);
void Bin2BCD(char);

//Timer Variables
char  milli_secs, seconds, minutes, hours, Flag, Key, MSD, MdD, LSD;


main ()
{
	LCDInit();

	IOInit();
	
	RegsInit();
    
    Timer23Init();              //Initialize the A/D converter
		
	IFS0bits.T3IF = 0;
	IEC0bits.T3IE = 1;			// Enable Timer2 & 3 overflow interrupt
	INTCON2 = 0x0000;

	DisplayStart();

	StopWatch();
}




Timer23Init()
{

    //##########################################	
    //  STEP 1:
	// 	Internal Clock (FOSC/2)	(FOSC = 8MHz)
	// 	1:1 prescale
	//	32 bit Timer2 and 3
    //##########################################	
    
    // ### Your Code Here ###   // ###
    
    		

    //##########################################
    //  STEP 2:
	// 	Load PR3 & PR2 with the count such that you get 1 tenth of a sec ticks. 
	//  Equation for PR calculation:  PR = (Fosc/2) / (# Ticks per second)
    //##########################################	
    
    // ### Your Code Here ###   // ###
    // ### Your Code Here ###   // ###			  


	TMR3 = 0x0000;  		
	TMR2 = 0x0000;  		

	T2CONbits.TON = 1; 		    // Start Timer
}




StopWatch()
{
	while(1)
	{
		Key = !GetKey();
		if(Key)
		{
			DisplayName();
			
			while(1)
			{
				Timer();
				Display(hours, minutes, seconds, milli_secs);

				Key = !GetKey();
				if(Key)
				break;
			}
		}
	}
}



IOInit()
{
	TRISDbits.TRISD6 = 1;	//  PORTD<6> as input, SW1.
}



unsigned char GetKey()
{
	if(PORTDbits.RD6)
		return 1; //TRUE;
	else
	{
		while(!PORTDbits.RD6);
		DebounceDelay();
		return 0; //FALSE;
	}
}



Timer()
{
	if(Flag)
	{
		Flag = 0;					// Clear Timer overflow flag
		milli_secs ++;

		if(milli_secs >= 10)
		{
			seconds += 1;				
			milli_secs = 00;

			if (seconds >= 60)      // seconds has reached 60?
			{						// YES
				minutes +=1;		// Increment minutes register
				seconds = 00;		// Clear seconds counter	
		
				if(minutes >= 60)	// Minutes counted 60?	
				{					// YES	
					hours += 1;		// Increment Hours	
					minutes = 0;	// Clear Minutes counter
					
					if(hours >= 99)
						hours = 0;
				}					// End of if(minutes >= 60)
			}	  					// End of if(seconds >= 60)		
		}	  						// End of if(milli_secs >= 6)		
	}		
}




DebounceDelay()
{
	unsigned long int i = 0x000FF;
	while(i--);    // 1.11 Seconds @ 20 MHz clock speed.
}



RegsInit()
{
	milli_secs = 0;
	seconds = 0;
	minutes = 0;
	hours = 0;
	Flag = 0;
	Key = 0;
}



void Display(char hours, char minutes, char seconds, char milli_secs)
{
	LCDL2Home();

	Bin2BCD(hours);
	LCDPut(MdD+0x30);
	LCDPut(LSD+0x30);
	LCDPut(':');

	Bin2BCD(minutes);
	LCDPut(MdD+0x30);
	LCDPut(LSD+0x30);
	LCDPut(':');

	Bin2BCD(seconds);
	LCDPut(MdD+0x30);
	LCDPut(LSD+0x30);
	LCDPut(':');

	Bin2BCD(milli_secs);
	LCDPut(LSD+0x30);
}



void Bin2BCD(char temp)                 // Reciev the byte in RAM(temp)
{
	LSD = temp;                     // Store the byte in LSD

   	for (MSD = 0 ; LSD >= 100 ; LSD>=100?(LSD -= 100):LSD, MSD++ ); // Do repeated subtraction with 100 untill LSD > 100, count interation(MSD)
   	for (MdD = 0 ; LSD >= 10  ; LSD>=10?(LSD -= 10):LSD,  MdD++);  // Do repeated subtraction with 10 untill LSD > 10, count interation(MsD)
}



void __attribute__((interrupt, shadow,no_auto_psv)) _T3Interrupt()
{
	IFS0bits.T3IF = 0;  //Clear Timer3 interrupt flag bit 
	Flag = 1;
}


DisplayStart()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('P');
	LCDPut('r');
	LCDPut('e');
	LCDPut('s');
	LCDPut('s');
	LCDPut(' ');
	LCDPut('S');
	LCDPut('3');
	LCDPut(' ');
	LCDPut('-');
	LCDPut(' ');
	LCDPut('S');
	LCDPut('t');
	LCDPut('a');
	LCDPut('r');
	LCDPut('t');
}

DisplayName()
{
	LCDClear();
	LCDHome();
	LCDPut('H');
	LCDPut('r');
	LCDPut(' ');
	LCDPut('M');
	LCDPut('n');
	LCDPut(' ');
	LCDPut('S');
	LCDPut('c');
	LCDPut(' ');
	LCDPut('S');
	LCDPut('/');
	LCDPut('1');
	LCDPut('0');
}
